package reportscollector

import (
	"compress/gzip"
	"encoding/xml"
	"io"
	"net/http"
	"time"
)

type dmarcRecord struct {
	Row struct {
		SourceIP        string `xml:"source_ip"`
		Count           int    `xml:"count"`
		PolicyEvaluated struct {
			Disposition string `xml:"disposition"`
			DKIM        string `xml:"dkim"`
			SPF         string `xml:"spf"`
		} `xml:"policy_evaluated"`
	}
	Identifiers struct {
		EnvelopeFrom string `xml:"envelope_from"`
		HeaderFrom   string `xml:"header_from"`
	} `xml:"identifiers"`
	AuthResults struct {
		DKIM struct {
			Domain string `xml:"domain"`
			Result string `xml:"result"`
		} `xml:"dkim"`
		SPF struct {
			Domain string `xml:"domain"`
			Result string `xml:"result"`
		} `xml:"spf"`
	} `xml:"auth_results"`
}

func (r *dmarcRecord) isFailure() bool {
	return (r.AuthResults.DKIM.Result == "fail" ||
		r.AuthResults.SPF.Result == "fail")
}

type dmarcPolicy struct {
	Domain string  `xml:"domain"`
	ADKIM  string  `xml:"adkim"`
	ASPF   string  `xml:"aspf"`
	P      string  `xml:"p"`
	SP     string  `xml:"sp"`
	PCT    float64 `xml:"pct"`
	FO     float64 `xml:"fo"`
}

type dmarcReport struct {
	Name xml.Name `xml:"feedback"`

	Version  string `xml:"version"`
	Metadata struct {
		ReportID     string `xml:"report_id"`
		Organization string `xml:"org_name"`
		Email        string `xml:"email"`
		DateRange    struct {
			StartSecs int64 `xml:"begin"`
			EndSecs   int64 `xml:"end"`
		} `xml:"date_range"`
	} `xml:"report_metadata"`

	Policy  *dmarcPolicy   `xml:"policy_published"`
	Records []*dmarcRecord `xml:"record"`
}

type DMARCHandler struct{}

func (h *DMARCHandler) Parse(contentType string, req *http.Request) ([]Event, error) {
	var r io.Reader
	switch contentType {
	case "text/xml":
		r = req.Body
	case "application/gzip":
		var err error
		r, err = gzip.NewReader(req.Body)
		if err != nil {
			return nil, err
		}
	default:
		return nil, ErrNoMatch
	}

	var report dmarcReport
	if err := xml.NewDecoder(r).Decode(&report); err != nil {
		return nil, err
	}

	var events []Event
	for _, rec := range report.Records {
		if rec.isFailure() {
			events = append(events, h.eventFromRecord(&report, rec))
		}
	}
	return events, nil
}

func (h *DMARCHandler) eventFromRecord(report *dmarcReport, rec *dmarcRecord) Event {
	e := make(Event)
	e.Set("type", "dmarc")
	e.Set("event_timestamp", time.Unix(report.Metadata.DateRange.EndSecs, 0))
	e.Set("domain", report.Policy.Domain)
	e.Set("report_id", report.Metadata.ReportID)
	e.Set("report_organization", report.Metadata.Organization)

	e.Set("dmarc_envelope_from", rec.Identifiers.EnvelopeFrom)
	e.Set("dmarc_header_from", rec.Identifiers.HeaderFrom)
	e.Set("dmarc_dkim", rec.AuthResults.DKIM.Result)
	e.Set("dmarc_spf", rec.AuthResults.SPF.Result)
	e.Set("failed_session_count", rec.Row.Count)

	return e
}
